
import sbt._
import Keys._
import play.sbt.PlayImport._
import PlayKeys._
import com.typesafe.sbt.SbtNativePackager._
import com.typesafe.sbt.SbtNativePackager.autoImport._
import NativePackagerKeys._
import sbt.Project.projectToRef


object ApplicationBuild extends Build {

  val appName = "%CPN%"
  val appVersion = "0.1"

  val prepareTaskKey = TaskKey[Unit]("prepare", "Creates the intermediate(xml) file")
  val generateTaskKey = TaskKey[Unit]("generate", "Create code files")


  def filePathMap(f: File) = (f.***) pair relativeTo(f.getParentFile)

  lazy val root = sbt.Project(appName, file("."))
    .enablePlugins(play.sbt.PlayScala)
    .settings(commands ++= Seq()
      ++ com.iteration3.relax.rsbt.Commands.list
      ++ com.iteration3.relax.rsbt.ComponentCommands.list
      ++ com.iteration3.relax.rsbt.LibCommands.list
      ++ com.iteration3.relax.rsbt.RelaxCommands.list)
    .settings(version := appVersion)
    .settings(onLoad in Global := com.iteration3.relax.rsbt.Commands.initChecks)
    .settings((stage in Universal) <<= (stage in Universal) map (_ => {com.iteration3.relax.rsbt.Commands.createSmileStarter(); file("todo")}))
    .settings(bashScriptExtraDefines += """addJava "-Dsmile.staging=true"""")
    .settings(bashScriptConfigLocation := Some((baseDirectory.value / "conf" / "jvmopts").getAbsolutePath))
    .settings(mappings in Universal <++= (packageBin in Compile) map { _ =>
      Seq()
    })
    .settings(com.iteration3.relax.rsbt.RootProjectTasks.taskList(prepareTaskKey, generateTaskKey): _*)
    .settings(hardCodedSettings: _*)
    .settings(SbtSettings.commonProjectSettings: _*)
    .aggregate()
    .dependsOn()






  def commonCommands =
    Seq(
      commands ++= Seq(
        com.iteration3.relax.rsbt.Commands.clearConsoleCommand,
        com.iteration3.relax.rsbt.Commands.cmdCommand,
        com.iteration3.relax.rsbt.Commands.addSwaggerCommand,
        com.iteration3.relax.rsbt.Commands.removeSwaggerCommand,
        com.iteration3.relax.rsbt.Commands.smileUpdateCommand)
    )

  def hardCodedSettings = commonCommands ++ hardCodedLibrarySettings ++ unmanagedJarSettings ++ artifactNameSettings

  def hardCodedLibrarySettings = libraryDependencies ++= hardCodedCommonDependencies


  def artifactNameSettings = artifactName := { (sv: ScalaVersion, module: ModuleID, artifact: Artifact) =>
    val classifierStr = artifact.classifier match {
      case None => "";
      case Some(c) => "-" + c
    }
    artifact.name + "-" + module.revision + classifierStr + "." + artifact.extension
  }

  def unmanagedJarSettings = unmanagedJars in Compile ++= {
    val mayBeRelaxRelaxHome = Option(System.getProperty("SMILE_HOME"));
    val relaxHome = file(mayBeRelaxRelaxHome.get);
    val dirs = (relaxHome / "lib" / "run-lib") +++ (relaxHome / "dist" / "run-dist")
    (dirs ** "*.jar").classpath
  }

  lazy val hardCodedCommonDependencies = Seq(

    "org.fusesource.jansi" % "jansi" % "1.4",

    "org.slf4j" % "slf4j-api" % "1.7.6",
    "org.slf4j" % "jul-to-slf4j" % "1.7.6",
    "org.slf4j" % "jcl-over-slf4j" % "1.7.6",

    "ch.qos.logback" % "logback-core" % "1.1.7",
    "ch.qos.logback" % "logback-classic" % "1.1.7",

    "com.typesafe.slick" % "slick_2.11" % "2.1.0",
    "com.typesafe.play" %% "play" % "2.4.11",

    "mysql" % "mysql-connector-java" % "6.0.4",
    "com.jolbox" % "bonecp" % "0.8.0.RELEASE",
    "org.scalikejdbc" %% "scalikejdbc" % "2.4.2",
    "com.google.inject" % "guice" % "3.0",

    "com.typesafe.akka" %% "akka-actor" % "2.3.8",
    "com.typesafe.akka" %% "akka-slf4j" % "2.3.8",
    "com.typesafe.akka" %% "akka-remote" % "2.3.8",
    "com.typesafe.akka" %% "akka-testkit" % "2.3.8",
    //  "com.typesafe.akka" %% "akka-persistence-experimental" % "2.3.8",

    "org.mongodb" % "mongo-java-driver" % "2.14.2",
    "org.scalatest" % "scalatest_2.11" % "2.2.0",
    "org.scalacheck" %% "scalacheck" % "1.10.1",
    "junit" % "junit" % "4.10" % "test",
    "org.pegdown" % "pegdown" % "1.1.0" % "test",
    "org.apache.httpcomponents" % "httpclient" % "4.5.2"

  )

}


trait DefaultSbtSettings {

  def commonProjectSettings: Seq[Setting[_]] = Seq();
  def commonScalaJsProjectSettings: Seq[Setting[_]] = Seq();


}
